; Updates various animation-related counters each frame

; Number of 1/60 frames since last time
zp_res anim_new_frames

; Variable animation rate

; Number of anim frames that should be generated/skipped
zp_res  anim_count

; Rate at which animation runs; affects anim_count
zp_res  anim_rate,2

; Incremented each anim frame
anim_frame = anim_accum + 1

; Beat

; Incremented by $10000 each beat
zp_res anim_beat,3

; Negative if new beat occurred
bss_res anim_new_beat

; Rate of beats. If synced_anim_beat is non-zero, this is updated with
; the current synced tempo each beat.
zp_res  anim_beat_rate,2

; If non-zero, beat is set to synced tempo
synced_anim_beat = tempo_stable

; anim_hue + anim_brt
zp_res anim_color

bss_res brt_mask_
bss_res brt_mask
bss_res anim_brt_ptr_,2
zp_res  anim_brt_ptr,2

; Brightness to use for pattern, might also have some hue bits set
zp_res anim_brt

; For zeroing hue bits in anim_brt; anim_brt is masked with this
; after being calculated
bss_res brt_color_mask

; Sets brt cycling mode, or does nothing if out of range
; In: Y = new mode
set_brt_mode:
	cpy #@ptr_count
	bge :+
	mov       brt_mask_,{@masks,y}
	mov anim_brt_ptr_+0,{@ptr_l,y}
	mov anim_brt_ptr_+1,{@ptr_h,y}
:       rts

@masks: .byte            $07,      $0F,      $3F,      $07,      $0F,      $3F
split_words @ptr, {@brt_pal0,@brt_pal1,@brt_pal2,@brt_pal3,@brt_pal4,@brt_pal5}

@brt_pal0:
@brt_pal1:
	.byte $27,$16,$05,$F4,$E3,$D2,$C1,$B0
	.byte $B0,$C1,$D2,$E3,$F4,$05,$16,$27
@brt_pal2:
   .repeat 8,i
	.repeat 8
	    .byte (i ^ $07)*$21
	.endrepeat
    .endrepeat
@brt_pal3:
@brt_pal4:
	.byte $B0,$C1,$D2,$E3,$F4,$05,$16,$27
	.byte $27,$16,$05,$F4,$E3,$D2,$C1,$B0
@brt_pal5:
    .repeat 8,i
	.repeat 8
	    .byte i*$21
	.endrepeat
    .endrepeat

; Uses newly set brt mode immediately
use_brt_mode:
	mov brt_mask,brt_mask_
	movw anim_brt_ptr,anim_brt_ptr_
	rts

; Calculates anim_brt
; In: A = offset (0 = normal)
; Out: A = anim_brt
calc_anim_brt:
	sta temp
	lda anim_beat + 2
	lsr a
	eor anim_beat + 1
	and #$0F
	eor anim_beat + 1
	ror
	ror
	ror
	ror
	lsr
	clc
	adc temp
	and brt_mask
	tay
	lda (anim_brt_ptr),y
	and brt_color_mask
	rts

bss_res tempo_mode

set_tempo_mode:
	sta tempo_mode
	
	tax
	mov brt_color_mask,{@masks,x}
	ldy @modes,x
	jmp set_brt_mode

@masks: .byte $07,$07,$07,$07
	.byte $FF,$FF,$FF,$FF
	.byte $07,$07,$07,$07
	.byte $FF,$FF,$FF,$FF

@modes: .byte 0, 1, 2, 2
	.byte 0, 1, 2, 2
	.byte 3, 4, 5, 5
	.byte 3, 4, 5, 5
	
zp_res anim_hue_pos,2
bss_res anim_hue_base
zp_res anim_hue_ptr,2
bss_res anim_hue_mask

; Current hue mode
bss_res hue_mode

; Rate of hue change
zp_res  anim_hue_rate

; Hue to use in patterns; low 3 bits always clear
zp_res anim_hue

; Sets hue cycling mode, or does nothing if mode is out of range
; In: Y = new mode
set_hue_mode:
	cpy #@ptr_count
	bge @bad
	lda hue_mode
	cmp #@ptr_count - 1
	bne :+
	mov anim_hue_base,anim_hue
:       sty hue_mode
	mov anim_hue_pos + 1,#0
	mov anim_hue_ptr+0,{@ptr_l,y}
	mov anim_hue_ptr+1,{@ptr_h,y}
	mov anim_hue_mask ,{@masks,y}
@bad:   rts

@masks: .byte           $1F,      $1F,      $3F
split_words @ptr, {@hue_pal0,@hue_pal1,@hue_pal2}

@hue_pal0:
    	.byte 0, 0, 0, 0, 8, 8, 8, 8, 16, 16, 16, 16, 8, 8, 8, 8
    	.byte 0, 0, 0, 0,-8,-8,-8,-8,-16,-16,-16,-16,-8,-8,-8,-8

@hue_pal1:
	.byte  0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 5*8, 6*8
	.byte  6*8, 6*8, 5*8, 5*8, 4*8, 3*8, 2*8, 1*8
	.byte -0*8,-1*8,-2*8,-3*8,-4*8,-5*8,-5*8,-6*8
	.byte -6*8,-6*8,-5*8,-5*8,-4*8,-3*8,-2*8,-1*8

@hue_pal2:
    .repeat $20,i
	.byte i * 8
    .endrepeat
    .repeat $20,i
	.byte ($1F - i) * 8
    .endrepeat

; Sets hue rate if index is valid
; In: Y = new rate
bss_res hue_rate_index

set_hue_rate:
	cpy #@count
	bge :+
	sty hue_rate_index
	mov anim_hue_rate,{@rates,y}
:       rts
@rates:
	.byte 0, $04, $08, $20, $40, $80, $FF
@count = * - @rates

bss_res anim_prev
zp_res  anim_accum,2

; Updates animation counters
animate_frame:
	; Calculate frames since last time. Can be 0,
	; but never twice in a row, so treat that as 1.
	lda nmi_count
	tay
	sec
	sbc anim_prev
	bne :+
	lda #1
	iny
:       sty anim_prev
	sta anim_new_frames
	
	; Add things for number of frames since last
	tay
	mov anim_new_beat,#0
	ldx anim_accum + 1
@frames:
	; anim_accum += anim_rate
	lda anim_accum
	clc
	adc anim_rate
	sta anim_accum
	txa
	adc anim_rate + 1
	tax

	; anim_hue_pos += anim_hue_rate
	lda anim_hue_pos
	clc
	adc anim_hue_rate
	sta anim_hue_pos
	bcc :+
	inc anim_hue_pos + 1
:
	; anim_beat += anim_beat_rate
	addw anim_beat,anim_beat_rate
	bcc @no_beat
	
	inc anim_beat + 2
	dec anim_new_beat
	
	jsr use_brt_mode

	; Only sync with tempo_accum on beats
	lda synced_anim_beat
	beq :+
	movw anim_beat_rate,tempo_rate
	movw anim_beat,tempo_accum
	mov anim_beat+2,tempo_beat_count
:

@no_beat:
	dey
	jne @frames

	; anim_count
	txa
	sec
	sbc anim_accum + 1
	sta anim_count
	stx anim_accum + 1
	
	; anim_brt
	lda #0
	jsr calc_anim_brt
	sta anim_brt

	; anim_hue
	lda anim_hue_pos + 1
	and anim_hue_mask
	tay
	lda (anim_hue_ptr),y
	clc
	adc anim_hue_base
	sta anim_hue
	
	; anim_color
	clc
	adc anim_brt
	sta anim_color
	
	rts

; Initializes module
init_animation:
	
	lda #0
	sta anim_accum
	sta anim_accum + 1
	sta anim_beat
	sta anim_beat + 1
	sta anim_beat + 2
	sta anim_hue_pos
	sta anim_hue_pos + 1
	sta anim_hue_rate
	sta anim_hue
	sta anim_hue_base
	
	movw anim_rate,#$100 / 2
	movw anim_beat,tempo_accum
	movw anim_beat_rate,tempo_rate
	mov brt_color_mask,#$07
	
	ldy #0
	jsr set_hue_mode
	
	ldy #0
	jsr set_brt_mode
	
	jsr use_brt_mode
	
	ldx nmi_count
	dex
	stx anim_prev
	
	rts
