; Controller reading and interpretation

; Button masks
joy_a           = $01
joy_b           = $02
joy_select      = $04
joy_start       = $08
joy_up          = $10
joy_down        = $20
joy_left        = $40
joy_right       = $80

zp_res  joy_held        ; currently held
zp_res  joy_pressed     ; just pressed
bss_res joy_released    ; just released

bss_res joy_repeat      ; used for auto-repeat

; Updates joy_ variables
; In: A = currently held buttons
; Preserved: A, X, Y
update_joy:
	pha
	eor joy_held
	sta joy_pressed
	and joy_held
	sta joy_released
	eor joy_pressed
	sta joy_pressed
	pla
	sta joy_held
	rts

; When a direction is held, sets joy_pressed periodically.
; Properly handles subtleties of auto-repeat.
; Should be called 60 times per second.
repeat_joy:
	lda joy_pressed
	beq :+
	mov joy_repeat,#20
:
	lda joy_repeat
	beq @ret

	dec joy_repeat
	bne @ret

	lda joy_held
	and #joy_up | joy_down | joy_left | joy_right
	sec
	sbc #1
	and joy_held
	bne @ret
	
	lda joy_held
	and #joy_up | joy_down | joy_left | joy_right
	sta joy_pressed
	mov joy_repeat,#6
@ret:   rts

; Reads controller into A and temp.
; Unreliable if DMC is playing.
; Preserved: X, Y
read_joy:
	lda #1
	sta $4016
	lsr a
	sta $4016
	
	; Read 8 bits
	ror a
:       pha
	lda $4016
	
	; Merge bits 0 and 1 into carry. Normal
	; controllers use bit 0, and Famicom
	; external controllers use bit 1.
	and #$03
	cmp #$01
	
	pla
	ror a
	bcc :-

	rts
