; Interprets controller when running litewall

bss_res paused
bss_res inversion
bss_res cur_pat_index

main_loop:
	
.if DEBUG
	movw pat_ptr,#pat_pinwheel
	jsr init_pat
.endif

@loop:
	jsr do_frame
	jsr handle_joy
	jmp @loop

; Handles controller input
handle_joy:
	lda lw_rand_seed
	eor random_bits
	sta random_bits
	
	lda joy_pressed
	lsr a
	jcs handle_a
	lsr a
	jcs handle_b
	lsr a
	jcs handle_select
	lsr a
	jcs handle_start
	lsr a
	bcs @up
	lsr a
	bcs @down
	lsr a
	bcs @left
	lsr a
	bcs @right
	rts

	; U D: Adjust anim rate
@up:    lda anim_rate+1
	cmp #2
	bge :+
	lda anim_rate
	clc
	adc #$20
	sta anim_rate
	bcc :+
	inc anim_rate + 1
:       rts

@down:  lda anim_rate
	cmp #$21
	lda anim_rate+1
	sbc #0
	blt :+
	lda anim_rate
	sec
	sbc #$20
	sta anim_rate
	bcs :+
	dec anim_rate + 1
:       rts

	; L R: Select pattern
@left:  ldx cur_pat_index
	bne :+
	ldx sram_pat_list_len
:       dex
	jmp change_pattern_list

@right: ldx cur_pat_index
	inx
	cpx sram_pat_list_len
	blt :+
	ldx #0
:       jmp change_pattern_list

change_pattern_list:
	stx cur_pat_index
	txa
	jsr get_pat_ptr
	jmp change_pattern_addr
	
; In: addr = pattern
change_pattern_addr:
	; If transition is already running, change immediately
	lda trans_ptr+1
	beq :+
	jsr init_pat_addr
	rts
:       
	jsr init_trans_pat
	jsr init_pat_addr
	ldy #1
	lda (pat_ptr),y
	and #>pat_info_notrans
	bne :+
	jsr init_rand_trans
:       rts

; B: Shape
handle_b:
bss_res shape_base

	mov shape_base,#0
	
@loop:
	; A+B: memory
	lda joy_pressed
	and #joy_a
	beq :+
	jsr handle_ab
	jmp @next
:
	lda joy_pressed
	and #joy_select
	beq :+
	mov shape_base,#8
:
	lda shape_base
	ora #7
	tax
	
	lda joy_pressed
	ldy #4
:       asl a
	bcs @shape
	dex
	dey
	bne :-
	jmp @next
	
@shape:
	txa
	and #$0F
	jsr set_litewall_shape
	
@next:
	jsr do_frame
	lda joy_held
	and #joy_b
	bne @loop
	
	rts

; Start: Tempo
handle_start:

bss_res new_tempo_mode
bss_res new_tempo_options

	mov new_tempo_mode,#-1
	mov new_tempo_options,#-1
	
@loop:
.if DEBUG
	lda joy_held
	eor #$FF
	and #joy_a | joy_b
	jeq forever
.endif

	; Select: Pause/Resume
	lda joy_pressed
	and #joy_select
	beq :+
	jsr handle_pause
:
	; U D: Adjust
	lda joy_pressed
	and #joy_up
	beq :+
	mov synced_anim_beat,#0
	ldx anim_beat_rate+1
	cpx #16
	bge :+
	inx
	inx
	stx anim_beat_rate+1
:
	lda joy_pressed
	and #joy_down
	beq :+
	mov synced_anim_beat,#0
	ldx anim_beat_rate+1
	cpx #3
	blt :+
	dex
	dex
	stx anim_beat_rate+1
:
	; L: Options
	lda joy_pressed
	and #joy_left
	beq :+
	inc new_tempo_options
:
	; R: mode
	lda joy_pressed
	and #joy_right
	beq :+
	inc new_tempo_mode
:
	jsr do_frame
	lda joy_held
	and #joy_start
	bne @loop
	
	lda new_tempo_mode
	cmp #3
	blt :+
	lda tempo_mode
	and #$03
:       sta temp
	
	lda new_tempo_options
	cmp #4
	blt :+
	lda tempo_mode
	lsr
	lsr
:       asl
	asl
	ora temp
	jsr set_tempo_mode

	rts

handle_pause:
	lda paused
	eor #$FF
	sta paused
	
; TODO: how to eliminate occasional glitch when unpausing?
.if 0
	and synced_anim_beat
	beq :+
	;movw anim_beat_rate,tempo_rate
	;movw anim_beat_,tempo_accum
	;mov anim_prev,tempo_prev
:
.endif
	rts

zp_res bg_hue

set_inversion:
	pha
	jsr confirm_litewall_update
	pla
	ldx #cols * rows - 1
:       sta lw_palsel,x
	dex
	bpl :-
set_background:
	lsr a
	lda bg_hue
	bcs :+
	lda #$3F
:       eor #$30
	jsr set_litewall_bgcolor
	jsr request_litewall_update
	rts

; Select: Hue
handle_select:

bss_res new_hue_mode

	mov new_hue_mode,#-1
	
@loop:
	; A B: Inversion
	lda inversion
	ora joy_pressed
	ora joy_released
	eor joy_released
	and #$03
	cmp inversion
	beq :+
	sta inversion
	jsr set_inversion
:

	; U D: Adjust
	lda joy_pressed
	and #joy_up
	beq :+
	ldy hue_rate_index
	iny
	jsr set_hue_rate
:

	lda joy_pressed
	and #joy_down
	beq :+
	ldy hue_rate_index
	dey
	jsr set_hue_rate
:

	; L: Hue mode
	lda joy_pressed
	and #joy_left
	beq :+
	inc new_hue_mode
	ldy new_hue_mode
	jsr set_hue_mode
:
	; R: Hue fine-tune
	lda joy_pressed
	and #joy_right
	beq :+
	lda anim_hue_base
	clc
	adc #$10
	sta anim_hue_base
:
	; Start: Cycle inversion background hue
	lda joy_pressed
	and #joy_start
	beq :++
	ldx bg_hue
	dex
	bpl :+
	ldx #$0C
:       stx bg_hue
	lda inversion
	jsr set_background
:
	jsr do_frame
	lda joy_held
	and #joy_select
	jne @loop

	rts

; A: Pattern
handle_a:
	bss_res handle_a_pat
	bss_res handle_a_pat_prev
	
	mov handle_a_pat_prev,#-1
	mov handle_a_pat,#0
	
@loop:
	lda joy_pressed
	and #joy_b
	beq :+
	jsr handle_ab
	jmp @next
:
	lda joy_pressed
	ldx #3
:       asl a
	bcs @pat
	dex
	bpl :-
	
	lda joy_pressed
	and #joy_select
	beq @next
	jsr rand_pat_index
	tax
	jsr change_pattern_list
	jmp @next

@pat:   lda #0
	cpx handle_a_pat_prev
	bne :+
	lda handle_a_pat
	eor #$04
:       sta handle_a_pat
	stx handle_a_pat_prev
	
	txa
	clc
	adc handle_a_pat
	tax
@pat_:  
	txa
	jsr get_fixed_pat_ptr
	jsr change_pattern_addr
	
	
@next:
	
	jsr do_frame
	lda joy_held
	and #joy_a
	bne @loop
	
	rts

; A+B: Memory
handle_ab:
	bss_res memorize
	
	mov memorize,#0
	
@loop:
	; Select: memorize
	lda joy_pressed
	and #joy_select
	ora memorize
	sta memorize

	lda joy_pressed
	ldx #3
:       asl a
	bcs @dpad
	dex
	bpl :-
	
	jmp @next

@dpad:  lda memorize
	beq :+
	mov memorize,#0
	jsr @memorize
	jmp @next
:       jsr @recall

@next:
	jsr do_frame
	lda joy_held
	and #joy_a | joy_b
	cmp #joy_a | joy_b
	beq @loop
	
	rts

@memorize:
	mov {mem_tempo_mode,x},tempo_mode
	mov {mem_hue_rate_index,x},hue_rate_index
	mov {mem_hue_mode,x},hue_mode
	mov {mem_pat_ptr0,x},pat_ptr+0
	mov {mem_pat_ptr1,x},pat_ptr+1
	mov {mem_anim_rate0,x},anim_rate+0
	mov {mem_anim_rate1,x},anim_rate+1
	mov {mem_anim_hue_base,x},anim_hue_base
	mov {mem_inversion,x},inversion
	mov {mem_bg_hue,x},bg_hue
	rts

@recall:
zp_res mem_slot
	lda mem_pat_ptr1,x
	bmi :+
	rts     ; not set yet
:       stx mem_slot
	ldy mem_pat_ptr0,x
	cpy pat_ptr+0
	bne :+
	cmp pat_ptr+1
	beq @same_pat
:       sty addr+0
	sta addr+1
	jsr change_pattern_addr
@same_pat:
	
	ldx mem_slot
	lda mem_tempo_mode,x
	jsr set_tempo_mode
	
	ldx mem_slot
	ldy mem_hue_rate_index,x
	jsr set_hue_rate
	
	ldx mem_slot
	ldy mem_hue_mode,x
	jsr set_hue_mode
	
	ldx mem_slot
	mov anim_rate+0,{mem_anim_rate0,x}
	mov anim_rate+1,{mem_anim_rate1,x}
	mov anim_hue_base,{mem_anim_hue_base,x}
	
	ldx mem_slot
	mov bg_hue,{mem_bg_hue,x}
	lda mem_inversion,x
	sta inversion
	jsr set_inversion
	
	jsr use_brt_mode
	
	rts

mem_slot_count = 4
bss_res mem_tempo_mode,mem_slot_count
bss_res mem_pat_ptr0,mem_slot_count
bss_res mem_pat_ptr1,mem_slot_count
bss_res mem_hue_rate_index,mem_slot_count
bss_res mem_hue_mode,mem_slot_count
bss_res mem_anim_rate0,mem_slot_count
bss_res mem_anim_rate1,mem_slot_count
bss_res mem_anim_hue_base,mem_slot_count
bss_res mem_inversion,mem_slot_count
bss_res mem_bg_hue,mem_slot_count

; Animates until joy changes
; Out: A = buttons held
do_frame:
@loop:
	; Deal with any changes
	
	; First try older one
	lda new_joy2
	cmp joy_held
	jne update_joy
	
	; They match, so use newer one AND overwrite old,
	; so that we don't user older one next time
	lda new_joy
	sta new_joy2
	cmp joy_held
	jne update_joy
	
	bit paused
	bmi @paused
	jsr gen_frame
	
	jmp @loop
	
@paused:
	jsr update_tempo
	
	; Wait one frame
	lda nmi_count
:       cmp nmi_count
	beq :-
	
	; Avoid animating anything
	ldx nmi_count
	dex
	stx anim_prev
	
	jmp @loop
