; Overall program flow from reset

cols = 10
lw_cols = 10 ; 8 or 10

; Two-byte queue of recent controller states. Only updated
; when controller changes. This allows code to accurately
; determine which button was held first when one is held
; and another pressed at the same time, even when main code
; isn't able to check controller every frame.
zp_res new_joy  ; current
zp_res new_joy2 ; previous

zp_res nmi_count,2      ; incremented every NMI

.include "lw_interface.s"
.include "tempo.s"
.include "rendering.s"
.include "pattern.s"
.include "animation.s"
.include "transition.s"
.include "transforms.s"
.include "random.s"
.include "controller.s"
.include "catalog.s"
.include "controls.s"
.include "config.s"

main:
	jsr init_shell
	
	lda #sram_fixed_pats_len - 1
	jsr init_direct_pat_index
	
	jsr main_loop
:       jmp :-
	
; Initializes module
init_shell:
	jsr init_pat_list
	
	; Show configuration if any buttons are pressed
	jsr read_joy
	sta joy_held
	beq :+
	
	jsr config_fixed_pats
	jsr config_pat_list
	
:       jsr prepare_litewall
	rts

; Prepares everything for beginning litewall
prepare_litewall:
	jsr init_litewall
	lda #4
	jsr set_litewall_shape
	jsr init_tempo
	jsr init_pattern
	jsr init_rendering
	jsr init_animation
	jsr init_transition
	jsr init_pat
	lda #0
	jsr set_tempo_mode
	
	ldy #3
	jsr set_hue_rate
	
	ldy #2
	jsr set_hue_mode
	
	rts

; Previews pattern until button pressed
; In: addr = pointer to pattern
preview_pattern:
	lda addr+1
	pha
	lda addr
	pha
	jsr prepare_litewall
	pla
	sta pat_ptr
	pla
	sta pat_ptr+1
	jsr init_pat
	mov joy_held,#$FF       ; TODO: less hacky way to avoid phantom press
	mov lw_joy,#$FF
	mov new_joy,#$FF
	mov new_joy2,#$FF
:       jsr do_frame
	lda joy_pressed
	beq :-
	jsr stop_litewall
	rts

; Generates frame
gen_frame:
	jsr update_tempo
	jsr animate_frame
	bit trans_ptr+1
	bmi :+
	jsr render_frame
	jmp :++
:       jsr render_transition
:       rts

zp_res nmi_saved_a
zp_res nmi_saved_x
zp_res nmi_saved_y

nmi:    sta nmi_saved_a         ; 9
	stx nmi_saved_x
	sty nmi_saved_y
	
	jsr lw_nmi_15           ; 6
	
	incw nmi_count
	
	; Update new_joy and new_joy2
	lda lw_joy
	cmp new_joy
	beq @ret

	ldx new_joy
	stx new_joy2
	tax
	eor new_joy
	stx new_joy
	
	; See whether B was just pressed/released, and
	; Start didn't change
	and #$0A
	cmp #$02
	bne @ret
	
	; Update and see whether Start and B are now held
	txa
	and #$0A
	cmp #$0A
	bne @ret
	
	jsr tap_tempo
	
@ret:
	ldy nmi_saved_y
	ldx nmi_saved_x
	lda nmi_saved_a
	
	rti
