; List of available patterns, with backup to SRAM and validation
; Main list is one cycled through with left/right
; Direct list is one accessed with A + Up/Down/Left/Right

; Lists of patterns in SRAM
sram_pat_list_          = sram_begin
sram_pat_list_max       = 20
sram_fixed_pats_        = sram_begin + $100
sram_fixed_pats_len     = 9

warm_boot       = $700 + ADDR
sram_pat_list   = warm_boot + 1
sram_fixed_pats = sram_pat_list + sram_pat_list_max + 1

bss_res sram_pat_list_len

; Initializes module
init_pat_list:
	lda #$4C
	cmp warm_boot
	beq :+
	sta warm_boot
	jsr load_pat_lists
:       jsr verify_pat_lists
	rts

; Loads pattern lists from SRAM
load_pat_lists:
	ldx #sram_pat_list_max
:       lda sram_pat_list_,x
	sta sram_pat_list,x
	dex
	bpl :-
	
	ldx #sram_fixed_pats_len
:       lda sram_fixed_pats_,x
	sta sram_fixed_pats,x
	dex
	bpl :-
	
	rts

; Saves pat lists to SRAM
save_pat_lists:
	ldx #sram_pat_list_max
:       lda sram_pat_list,x
	sta sram_pat_list_,x
	dex
	bpl :-
	
	ldx #sram_fixed_pats_len
:       lda sram_fixed_pats,x
	sta sram_fixed_pats_,x
	dex
	bpl :-
	
	rts

; Sets pat_list_editor variables to use main pattern list
use_pat_list:
	mov pat_list_fixed_size,#0
	mov pat_list_max_size,#sram_pat_list_max
	movw pat_list_ptr,#sram_pat_list
	rts

; Sets pat_list_editor variables to use direct pattern list
use_direct_pat_list:
	mov pat_list_fixed_size,#1
	mov pat_list_max_size,#sram_fixed_pats_len
	movw pat_list_ptr,#sram_fixed_pats
	rts

; Verifies lists and resets if corrupt
verify_pat_lists:
	jsr use_direct_pat_list
	jsr pat_list_len
	beq set_default_pat_lists
	
	jsr use_pat_list
	jsr pat_list_len
	sta sram_pat_list_len
	beq set_default_pat_lists
	
	rts

; Clears lists to default
set_default_pat_lists:
	ldx #0
:       txa
	sta sram_pat_list,x
	inx
	cpx #patterns_size - 1
	bne :-
	
	mov sram_pat_list + patterns_size - 1,#-1
	
	ldx #0
:       txa
	sta sram_fixed_pats,x
	inx
	cpx #sram_fixed_pats_len - 1
	bne :-
	
	mov sram_fixed_pats + 5,#patterns_size - 1
	mov sram_fixed_pats + sram_fixed_pats_len - 1,#patterns_size - 1
	mov sram_fixed_pats + sram_fixed_pats_len,#-1
	
	jsr verify_pat_lists
	
	rts

; In: A = index
; Out: addr = pattern ptr
get_pat_ptr:
	cmp sram_pat_list_len
	blt :+
	internal_error 1
	lda #0
:       tax
	lda sram_pat_list,x
	cmp #patterns_size
	blt :+
	internal_error 2
	lda #0
:
get_pat_ptr_:
	asl
	asl
	tax
	lda patterns,x
	sta addr
	lda patterns+1,x
	sta addr+1
	
	rts

; In: A = index
; Out: addr points to pattern
get_fixed_pat_ptr:
	cmp #sram_fixed_pats_len
	blt :+
	internal_error 3
	lda #0
:       tax
	lda sram_fixed_pats,x
	cmp #patterns_size
	blt :+
	internal_error 4
	lda #0
:       jmp get_pat_ptr_

.macro def_pat Addr, Name
	.local Str
	.pushseg
	.rodata
	Str:
		.byte Name,0
	.popseg
	.word Addr, Str
.endmacro

patterns:
	def_pat pat_horiz_comb,         "Horizontal Comb"
	def_pat pat_vert_comb,          "Vertical Comb"
	
	def_pat pat_contracting_spiral, "Inward Spiral"
	def_pat pat_expanding_spiral,   "Outward Spiral"
	
	def_pat pat_snakes,             "Snakes on a NES"
	
	def_pat pat_falling_stars,      "Falling Stars"
	def_pat pat_stars,              "Stars"

	def_pat pat_vert_stripes,       "Vertical Stripes"
	def_pat pat_horiz_stripes,      "Horizontal Stripes"
	
	def_pat pat_pinwheel,           "Pinwheel"
	
	def_pat pat_round_tunnel,       "Round Tunnel"
	def_pat pat_square_tunnel,      "Rectangular Tunnel"
	
	def_pat pat_black,              "All black"
patterns_size = (* - patterns) / 4
