; Edits pattern list

; Point to your list
zp_res  pat_list_ptr,2

; Set to the max size
bss_res pat_list_max_size

; If non-zero, list size can't be changed
bss_res pat_list_fixed_size

pat_list_x      = 8
pat_list_y      = 6
pat_list_width  = 30 - pat_list_x

bss_res pat_list_cursor
bss_res pat_list_size
bss_res pat_list_recent ; simple clipboard

; Determines length of pattern list
; Out: A = 0 if invalid, flags set based on A
pat_list_len:
	ldy #0
:       lda (pat_list_ptr),y
	bmi :+
	cmp #patterns_size
	bge @invalid
	iny
	bpl :-
	jmp @invalid
	
:       tya
	cmp #2
	blt @invalid
	dey
	cpy pat_list_max_size
	blt :+
@invalid:
	lda #0
:       cmp #0
	rts

; Edits pattern list and returns when done      
edit_pat_list:
	jsr pat_list_len
	bne :+
	ldy #0
	mov {(pat_list_ptr),y},#0
	iny
	mov {(pat_list_ptr),y},#-1
	jmp edit_pat_list
:       sta pat_list_size

	jsr @init
	
:       jsr poll_joy
	
	jsr @handle_joy
	
	lda joy_pressed
	and #joy_select
	beq :-
	
	rts

@init:
	lda #pat_list_y - 2
	jsr move_to_line
	ora #3
	sta VADDR
	jsr draw_str
	
	jsr pat_list_draw
	
	lda #0
	jsr pat_list_move_cursor
	
	mov pat_list_recent,#0
	
	jsr config_enable_screen
	rts

@handle_joy:
	lda joy_pressed
	and #joy_up
	beq :+
	lda pat_list_cursor
	beq :+
	sec
	sbc #1
	jsr pat_list_move_cursor
:
	lda joy_pressed
	and #joy_down
	beq :+
	lda pat_list_cursor
	cmp pat_list_size
	bge :+
	clc
	adc #1
	cmp pat_list_max_size
	jlt pat_list_move_cursor
:
	lda joy_pressed
	and #joy_a
	beq :+
	lda pat_list_fixed_size
	bne :+
	jmp @insert_row
:
	lda joy_pressed
	and #joy_left
	beq :+
	lda #-1
	jmp @cycle_pattern
:
	lda joy_pressed
	and #joy_right
	beq :+
	lda #+1
	jmp @cycle_pattern
:
	lda pat_list_cursor
	cmp pat_list_size
	blt :+
	rts
:
	lda joy_pressed
	and #joy_b
	beq :+
	lda pat_list_fixed_size
	bne :+ 
	jmp @erase_row
:
	lda joy_pressed
	and #joy_start
	bne @preview
	
	rts

@preview:
	ldy pat_list_cursor
	lda (pat_list_ptr),y
	jsr get_pat_ptr_
	jsr preview_pattern
	
	jsr begin_update
	jsr load_palette
	jsr config_enable_screen
	mov joy_pressed,#0
	rts

@cycle_pattern:
	ldx pat_list_cursor
	cpx pat_list_size
	blt :+
	pha
	jsr @insert_row_
	pla
	jsr @cycle_pattern
	jmp pat_list_draw
	
:
	ldy pat_list_cursor
	clc
	adc (pat_list_ptr),y
	bpl :+
	lda #patterns_size - 1
:       cmp #patterns_size
	blt :+
	lda #0
:       sta pat_list_recent
	sta (pat_list_ptr),y
	lda pat_list_cursor
	jsr begin_update
	jsr pat_list_draw_entry
	jmp end_update

@insert_row_:
	ldy pat_list_size
	jmp @first
:       dey
	lda (pat_list_ptr),y
	iny
	sta (pat_list_ptr),y
	dey
@first: cpy pat_list_cursor
	bne :-
	inc pat_list_size
	jsr pat_list_size_changed
	rts

@insert_row:
	lda pat_list_size
	cmp pat_list_max_size
	blt :+
	rts
:       jsr @insert_row_
	ldy pat_list_cursor
	mov {(pat_list_ptr),y},pat_list_recent
	jsr pat_list_draw
	rts

@erase_row:
	lda pat_list_size
	cmp #3
	bge :+
	rts

:       ldy pat_list_cursor
	mov pat_list_recent,{(pat_list_ptr),y}
	jmp :++
:       lda (pat_list_ptr),y
	dey
	sta (pat_list_ptr),y
	iny
:       iny
	cpy pat_list_size
	bne :--
	
	dec pat_list_size
	jsr pat_list_size_changed
	jsr pat_list_draw

	jsr begin_update
	movw addr,#@empty_str
	lda pat_list_size
	jsr pat_list_draw_entry_
	jsr end_update
	
	lda pat_list_cursor
	cmp pat_list_size
	bne :+
	sec
	sbc #1
	jsr pat_list_move_cursor
:       rts

@empty_str:
	.byte 0

pat_list_move_cursor:
	pha
	jsr begin_update
	lda #' '
	jsr @draw
	pla
	sta pat_list_cursor
	lda #'>'
	jsr @draw
	jmp end_update

@draw:
	pha
	lda pat_list_cursor
	clc
	adc #pat_list_y
	jsr move_to_line
	ora #pat_list_x - 1
	sta VADDR
	pla
	sta VDATA
	rts

pat_list_size_changed:
	ldy pat_list_size
	mov {(pat_list_ptr),y},#-1
	rts

pat_list_draw:
	lda #0
@loop:  pha
	and #$01
	bne :+
	jsr begin_update
:       pla
	pha
	jsr pat_list_draw_entry
	jsr end_update
	pla
	clc
	adc #1
	cmp pat_list_size
	blt @loop
	rts

; In: A = entry (0 to pat_list_size - 1)
pat_list_draw_entry:
	pha
	tay
	lda (pat_list_ptr),y
	asl
	asl
	tay
	
	mov addr+0,{patterns+2,y}
	mov addr+1,{patterns+3,y}
	
	pla

pat_list_draw_entry_:
	clc
	adc #pat_list_y
	jsr move_to_line
	ora #pat_list_x
	sta VADDR
	
	lda #pat_list_width
	jsr draw_str_padded
	
	rts
