; Transitions based on wipes, using various patterns stored in tables

.macro table_trans Table
	ldy trans_pos+1
:       ldx Table,y
	lda old_colors,x
	sta new_colors,x
	dey
	bpl :-
.endmacro

.macro table_trans_reverse Table
	lda #127
	sec
	sbc trans_pos+1
	tay
:       ldx Table + cols*pitch - 128,y
	lda old_colors,x
	sta new_colors,x
	iny
	bpl :-
.endmacro

table_trans_init:
	movw trans_pos,#cols * pitch * $100 - 1
	rts

table_trans_anim:
	; Transition at constant rate, regardless of frame rate
	lda anim_new_frames
	asl
	eor #$FF
	sec
	adc trans_pos + 1
	sta trans_pos + 1
	jmi end_trans
	rts
	
; trans_expanding_spiral

trans_expanding_spiral:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans spiral
	jmp table_trans_anim

; trans_contracting_spiral

trans_contracting_spiral:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans_reverse spiral
	jmp table_trans_anim

; trans_wipe_down

trans_wipe_down:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans wipe_vert_table
	jmp table_trans_anim

; trans_wipe_up

trans_wipe_up:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans_reverse wipe_vert_table
	jmp table_trans_anim

wipe_vert_table:
    .byte $00,$08,$10,$18,$20,$28,$30,$38,$40,$48,$01,$09,$11,$19,$21,$29
    .byte $31,$39,$41,$49,$02,$0A,$12,$1A,$22,$2A,$32,$3A,$42,$4A,$03,$0B
    .byte $13,$1B,$23,$2B,$33,$3B,$43,$4B,$04,$0C,$14,$1C,$24,$2C,$34,$3C
    .byte $44,$4C,$05,$0D,$15,$1D,$25,$2D,$35,$3D,$45,$4D,$06,$0E,$16,$1E
    .byte $26,$2E,$36,$3E,$46,$4E,$07,$0F,$17,$1F,$27,$2F,$37,$3F,$47,$4F

; trans_wipe_left

trans_wipe_left:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans wipe_horiz_table
	jmp table_trans_anim

; trans_wipe_right

trans_wipe_right:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans_reverse wipe_horiz_table
	jmp table_trans_anim

wipe_horiz_table: ; kind of dumb table, but not worth special coding
    .byte $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0A,$0B,$0C,$0D,$0E,$0F
    .byte $10,$11,$12,$13,$14,$15,$16,$17,$18,$19,$1A,$1B,$1C,$1D,$1E,$1F
    .byte $20,$21,$22,$23,$24,$25,$26,$27,$28,$29,$2A,$2B,$2C,$2D,$2E,$2F
    .byte $30,$31,$32,$33,$34,$35,$36,$37,$38,$39,$3A,$3B,$3C,$3D,$3E,$3F
    .byte $40,$41,$42,$43,$44,$45,$46,$47,$48,$49,$4A,$4B,$4C,$4D,$4E,$4F

; trans_wipe_down_right

trans_wipe_down_right:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans wipe_diagonal_table
	jmp table_trans_anim

; trans_wipe_up_left

trans_wipe_up_left:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans_reverse wipe_diagonal_table
	jmp table_trans_anim

wipe_diagonal_table:
    .byte $00,$01,$08,$02,$09,$10,$03,$0A,$11,$18,$04,$0B,$12,$19,$20,$05
    .byte $0C,$13,$1A,$21,$28,$06,$0D,$14,$1B,$22,$29,$30,$07,$0E,$15,$1C
    .byte $23,$2A,$31,$38,$0F,$16,$1D,$24,$2B,$32,$39,$40,$17,$1E,$25,$2C
    .byte $33,$3A,$41,$48,$1F,$26,$2D,$34,$3B,$42,$49,$27,$2E,$35,$3C,$43
    .byte $4A,$2F,$36,$3D,$44,$4B,$37,$3E,$45,$4C,$3F,$46,$4D,$47,$4E,$4F

; trans_vert_comb

trans_vert_comb:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans @table
	jmp table_trans_anim

@table:
    .byte $00,$10,$20,$30,$40,$01,$11,$21,$31,$41,$02,$12,$22,$32,$42,$03
    .byte $13,$23,$33,$43,$04,$14,$24,$34,$44,$05,$15,$25,$35,$45,$06,$16
    .byte $26,$36,$46,$07,$17,$27,$37,$47,$0F,$1F,$2F,$3F,$4F,$0E,$1E,$2E
    .byte $3E,$4E,$0D,$1D,$2D,$3D,$4D,$0C,$1C,$2C,$3C,$4C,$0B,$1B,$2B,$3B
    .byte $4B,$0A,$1A,$2A,$3A,$4A,$09,$19,$29,$39,$49,$08,$18,$28,$38,$48

; trans_dissolve

trans_dissolve:
	.word 0
	.word table_trans_init
	.word @draw

@draw:  table_trans @table
	jmp table_trans_anim

@table:
    .byte $45,$0B,$1D,$34,$12,$0C,$29,$31,$3F,$4C,$04,$42,$35,$07,$2E,$27
    .byte $21,$4A,$0E,$1F,$30,$08,$15,$3D,$25,$14,$49,$4D,$4E,$09,$1B,$3B
    .byte $16,$05,$20,$36,$13,$2A,$23,$38,$1A,$17,$2D,$4F,$22,$41,$01,$46
    .byte $24,$18,$00,$1E,$48,$33,$32,$40,$3A,$06,$44,$10,$11,$47,$1C,$43
    .byte $2B,$0D,$2C,$39,$19,$02,$4B,$3E,$03,$28,$26,$0F,$2F,$0A,$37,$3C
