; Handles transitions between patterns

trans_info = 0
trans_init = 2
trans_draw = 4

zp_res trans_ptr,2      ; transition effect
zp_res pat2_ptr,2       ; old pattern being transitioned out
zp_res trans_pos,2      ; space for transition to store general position
zp_res trans_vars,10    ; space for transition-related variables

lw_colors2_stack_top = <(lw_colors2 + cols*pitch - 1)

new_colors = lw_colors
old_colors = lw_colors2

; Uses current pattern as starting pattern for transition
init_trans_pat:
	lda pat_ptr
	sta pat2_ptr
	lda pat_ptr+1
	sta pat2_ptr+1
	rts

; Begins transition. Old and new patterns must have been set.
init_trans:
	; Determine whether both patterns can run together
	ldy #0
	lda (pat2_ptr),y
	lsr a
	bne @concurrent ; stack works with anything
	lda (pat_ptr),y
	bcc @modify     ; modify must be frozen
	cmp #0
	bne @concurrent ; normal works with normal or stack
@modify:
	mov pat2_ptr+1,#0
	
	jsr save_lw_colors
@concurrent:
	movw trans_pos,#cols * pitch * $100 - 1
	
	ldy #trans_init
	jmp call_trans

; Immediately ends transition
end_trans:
	mov trans_ptr+1,#0
	rts

; Sends message Y to current transition
call_trans:
	jmp_proc trans_ptr

; Sends message Y to pat2
call_pat2:
	jmp_proc pat2_ptr

; Initializes module
init_transition:
	movw trans_ptr,#0
	rts

; Renders frame using current transition
render_transition:
	lda anim_count
	pha
	
	lda pat2_ptr + 1
	beq @frozen_old_pat
	
	; Draw old pattern to lw_colors2. Do this first, so that any
	; extra stack writes will write into lw_colors now, before
	; anything important is there.
	ldy #0
	lda (pat2_ptr),y
	lsr
	bne @all_stack
	
	jsr confirm_litewall_update
	ldy #pat_draw
	jsr call_pat2
	jsr save_lw_colors
	jmp :+
	
@all_stack:
	; No need to confirm update, since this is separate buffer
	mov stack_buffer,#lw_colors2_stack_top
	ldy #pat_draw
	jsr call_pat2
	mov stack_buffer,#lw_colors_stack_top
:
@frozen_old_pat:
	jsr confirm_litewall_update
	
	pla
	sta anim_count
	pha
	
	; Draw new pattern
	jsr draw_pat
	
	pla
	sta anim_count
	
	; Transition
	ldy #trans_draw
	jsr call_trans
	
	jsr request_litewall_update
	
	rts

; Quickly copies lw_colors to lw_colors2
save_lw_colors:
	tsx
	stx saved_s
	ldx #lw_colors2_stack_top
	txs
	
	; Cycles = cols*pitch*(9/@unroll + 7)
	@unroll = 8
	
	ldx #cols * pitch - 1
	sec
:       .repeat @unroll,i
		lda lw_colors - i,x
		pha
	.endrepeat
	
	txa
	sbc #@unroll
	tax
	bpl :-
	
	ldx saved_s
	txs
	rts
