; NES version of litewall
;
; ca65 -I nes -I common nes/litewall.s
; ld65 -C nes/nrom.cfg -o litewall.nes nes/litewall.o
 
;DEBUG = 1

; Location of battery-backed RAM
sram_begin = $6000

.include "neshw.inc"
.include "macros.inc"
.include "delay.s"
.include "common.inc"
.include "main.s"

reset:
	ldx #$FF
	txs
	
	jsr init_mmc1
	
	jsr clear_ram
	
    .if DEBUG
	jsr init_debug
    .endif
    
	jmp main

clear_ram:
	lda #0
	ldx #0
:       sta 0,x
	sta $200,x
	sta $300,x
	sta $400,x
	sta $500,x
	sta $600,x
	inx
	bne :-
	
	; Clear below stack
	tsx
	inx
:       dex
	sta $100,x
	bne :-
	
	rts

init_mmc1:
	; $xxx1 instead of $xxx0 to simplify testing on my devcart
	; Makes no difference to MMC1.
	
	; 32K PRG, horizontal mirroring
	lda #$47 << 1
:       sta $8001
	lsr a
	cmp #$40 >> 5
	bne :-
	
	; Enable WRAM
	lda #$40
:       sta $A001
	lsr a
	cmp #$40 >> 5
	bne :-
	
	lda #$40
:       sta $C001
	lsr a
	cmp #$40 >> 5
	bne :-
	
	lda #$40
:       sta $E001
	lsr a
	cmp #$40 >> 5
	bne :-
	
	rts

irq:    jmp irq
	
	
.segment "HEADER"
	.byte "NES",26, 2,1, $12,0 ; 32K PRG, 8K CHR, battery
	.byte 0,0,0,0,0,0,0,0

.segment "VECTORS"
	.word 0,0,0, nmi, reset, irq

.segment "CHARS"

.res $10,0
.res $200 - $10

.incbin "ascii.chr"

.include "shapes.s"
