; Core NMI that does rendering, and whatever variables it uses

row_height = 22
lw_cols_ = 10   ; number of rendered columns
lw_offset_ = (lw_cols_ - lw_cols)/2 * pitch

row_colors_size = row_height / 2
sp_res row_colors_,row_colors_size
row_colors = row_colors_ + row_colors_size - 1 - lw_cols_

bss_res lw_ready
zp_res  lw_ptr,2
zp_res  colors_page
zp_res  tints_page
zp_res  lw_colors_ptr,2
zp_res  lw_palsel_ptr,2
bss_res lw_bgcolor
zp_res  lw_tint_mask
bss_res lw_shape_select
bss_res lw_force_tints

seg_data BSS, .align 256

lw_copy_loop_size = 30

bss_res lw_copy_loop,lw_copy_loop_size
bss_res lw_colors_copy,lw_cols_ * pitch
bss_res lw_palsel_copy,lw_cols_ * pitch
bss_res row_tints,lw_cols_

lw_copy_loop_half = lw_cols*pitch / 2

zp_res lw_saved_s

.align 256
lw_nmi_15:      ; 15 cycles between NMI and reaching here
	delay 7
	
	lda nmi_sync_count
	and #$02
	beq :+
:
	lda #0                  ; 18
	sta $2003
	lda #>sprites
	sta $4014
	lda #$0
	sta $2001
	
	; do first, since it writes to stack
	delay 1730 - 14 - 18 - 1647 - 8 - 15 - 16 - 5 - 3 + 5 - 1
	
	; Copy if ready
	inc lw_ready            ; 1641 + 8
	beq @ready
	dec lw_ready
	delay -2 - 6 + 7 + lw_copy_loop_half*41
	jmp lw_copied
@ready:
	; Had to unroll to run fast enough
	ldx #lw_copy_loop_half - 1
	jmp lw_copy_loop ; RAM copy of loop below
lw_copy_loop_in_rom:
	@half = lw_copy_loop_half
:       lda $1234                      ,x
	sta lw_palsel_copy + lw_offset_,x
	lda $1234                      ,x
	sta lw_colors_copy + lw_offset_,x
	
	lda $1234                       + @half,x
	sta lw_palsel_copy + lw_offset_ + @half,x
	lda $1234                       + @half,x
	sta lw_colors_copy + lw_offset_ + @half,x
	
	dex
	bpl :-
	jmp :+
:
lw_copied:
	
	delay 1640 - lw_copy_loop_half*41
	
	tsx                     ; 5
	stx lw_saved_s
	
	; Toggle between lookup tables each frame
	lda colors_page         ; 16
	eor #(>color_lookup0) ^ (>color_lookup1)
	sta colors_page
	lda tints_page
	eor #(>tint_lookup0) ^ (>tint_lookup1)
	sta tints_page
	
	; Synchronize to one of two times, two cycles apart
	lda nmi_sync_count
	and #$02
	bne :+
:       inc nmi_sync_count      ; 5
	bit $2002
	bpl :+
	nop
	bit <0
:       

	; Set background color
	ldx #$3F                ; 72
	lda lw_bgcolor
	
	ldy #0
	sty $2001
	stx $2006
	sty $2006
	sta $2007
	
	ldy #$04
	stx $2006
	sty $2006
	sta $2007
	
	ldy #$08
	stx $2006
	sty $2006
	sta $2007
	
	ldy #$0C
	stx $2006
	sty $2006
	sta $2007
	
	lda #$10
	sta $2001
	
	; Read controller
	ldx #1                  ; 74
	stx $4016
	lda $4016
	dex
	stx $4016
	and #$FC
	tax
	cpx $4016
	ror a
	cpx $4016
	ror a
	cpx $4016
	ror a
	cpx $4016
	ror a
	cpx $4016
	ror a
	cpx $4016
	ror a
	cpx $4016
	ror a
	cpx $4016
	ror a
	eor #$FF
	sta lw_joy
	
	jmp :+
.align 256
:
	
	; Synchronize down to the cycle
	lda $2002       ; 12
	and #$20
	beq :+
:       beq :+
:       

zp_res tint_idx         ; row in left-hand column of squares
zp_res lw_row           ; rows of squares remaining
zp_res color_idx        ; row in right-hand column of squares

zp_res tint_remain      ; scanlines pairs of tint remainint
zp_res row_frac         ; 113.6666 delay fraction
	
	mov tint_idx,#-1 ; incremented extra time before being used
	
	; Clear address away from palette, otherwise occasional glitch
	; occurs in upper-left corner
	sta $2006
	sta $2006
	
	; Put some blank lines above squares (what a mess of additions)
	delay  -144 + -12 + -35 * lw_cols_ + top_border*341/3 + 38+5 + 1 + 16 - 8
	
	; Translate first row of colors
	ldx #<row_colors + lw_cols_ - 1
	txs
	lda #(lw_cols_-1)*pitch
:       tax
	lda lw_palsel_copy,x
	and #palsel_mask
	ora colors_page
	sta lw_ptr+1
	ldy lw_colors_copy,x
	lda (lw_ptr),y
	pha
	txa
	clc
	adc #-pitch
	bcs :-
	
	mov color_idx,#lw_cols_*pitch
	mov lw_row,#rows - 1
	jmp @next_row

.macro lw_triplet Pal0, Pal1, Pal2, Tint
	ldx tint_idx
	
	lda lw_palsel_copy+(Tint+0)*pitch,x
	and #palsel_mask
	ora tints_page
	sta lw_ptr+1
	ldy lw_colors_copy+(Tint+0)*pitch,x
	lda (lw_ptr),y
	and lw_tint_mask
	sta row_tints+Tint+0
	
	lda lw_palsel_copy+(Tint+1)*pitch,x
	and #palsel_mask
	ora tints_page
	sta lw_ptr+1
	ldy lw_colors_copy+(Tint+1)*pitch,x
	lda (lw_ptr),y
	and lw_tint_mask
	sta row_tints+Tint+1
	
    .if Pal2 < 10
 	ldy row_colors+Pal2
   .else
	lda lw_palsel_copy+(Tint+2)*pitch,x
	and #palsel_mask
	ora tints_page
	sta lw_ptr+1
	ldy lw_colors_copy+(Tint+2)*pitch,x
	lda (lw_ptr),y
	and lw_tint_mask
	sta row_tints+Tint+2
    .endif
	lda row_colors+Pal0
	ldx row_colors+Pal1
	bit $2007
	sta $2007
	stx $2007
    .if Pal2 < 10
	sty $2007
    .else
	bit $2007
    .endif
.endmacro

@next_row:
	; Disable rendering at lower-right corner of row
	
	bit $2002
	ldx #$3F                ; 16
	lda #0
	stx $2006
	sta $2001
	sta $2006
	
	ldx lw_saved_s          ; 15
	txs
	inc tint_idx
	lda tints_page
	eor #(>tint_lookup0) ^ (>tint_lookup1)
	sta tints_page
	
	delay 114 - 114
	
	; Update palette and translate tints
	
	lw_triplet 1, 2, 0, 0
	delay 113 - 110
	
	inc color_idx           ; 13
	lda colors_page
	eor #(>color_lookup0) ^ (>color_lookup1)
	sta colors_page
	
	mov tint_remain,#row_height / 2 ; 10
	mov row_frac,#0
	
	lw_triplet 3, 4, 9, 2
	delay 114 - 111
	
	lw_triplet 5, 6, 10, 4
	delay 114 - 111
	
	lw_triplet 7, 8, 10, 7
	
	delay 113 - 33 - 51 - 13 - 3
	
	; See whether this is last row and tint isn't used on it
	lda row_tints+0
	ora row_tints+1
	ora row_tints+2
	ora row_tints+3
	ora row_tints+4
	ora row_tints+5
	ora row_tints+6
	ora row_tints+7
	ora row_tints+8
	ora row_tints+9
	and #$E0
	ora lw_force_tints
	ora lw_row
	cmp #1
	
	lda lw_shape_select
	and #$10
	ora #$80
	sta $2000
	
	; Apply tint to squares and translate next row's colors
	ldx #<row_colors + lw_cols_
	txs
	
	; Scroll to position for current shape
	ldy #$08
	lda lw_shape_select
	and #$0F
	sta $2006
	lda lw_shape_select
	and #$E0
	sta $2006
	sty $2001
	ldy #2
	sty $2005
	
	ldx color_idx
	
	bge @first_tint_row
	jmp @skip_final_row

.macro lw_tint_row
	lda row_tints+0 ; 80
	sta $2001
	lda row_tints+1
	sta $2001
	lda row_tints+2
	sta $2001
	lda row_tints+3
	sta $2001
	lda row_tints+4
	sta $2001
	lda row_tints+5
	sta $2001
	lda row_tints+6
	sta $2001
	lda row_tints+7
	sta $2001
	lda row_tints+8
	sta $2001
	lda row_tints+9
	sta $2001
.endmacro

@next_tint_row:
	lda lw_palsel_copy,x; 13
	and #palsel_mask
	ora A:colors_page
	sta lw_ptr+1
	
	lda row_frac    ; 10
	adc #86         ; works regardless of carry
	bcc :+
:       sta row_frac
	
@first_tint_row:
	lw_tint_row     ; 80
	
	ldy lw_colors_copy,x; 20
	lda (lw_ptr),y
	pha
	txa
	sec
	sbc #pitch
	tax
	
	delay 3
	
	lda row_frac    ; 10
	adc #86         ; works regardless of carry
	bcc :+
:       sta row_frac
	
	lw_tint_row     ; 80
	
	dec tint_remain ; 10
	beq :+
	jmp @next_tint_row
:
	dec lw_row
	bmi :+
	jmp @next_row
:

@skip_final_row:
	bit $2002
	
	ldx lw_saved_s
	txs
	
	rts
