; Displays 10x8 grid of colored squares, each with a settable color.

; Things that can be changed
.ifndef lw_cols
    lw_cols = 10        ; number of columns, <= 10
.endif
.ifndef rows
    rows = 8    ; number of rows, <= 8, lower gives more time outside of NMI
.endif
pitch = rows    ; number of bytes to next column, must be >= rows
top_border = 4  ; >= 4, higher gives less time outside NMI

bss_res lw_joy          ; Currently pressed buttons, updated every NMI
bss_res lw_rand_seed    ; Incremented when waiting for update

.include "nmi_sync.s"
.include "lw_core.s"

lw_max_shapes = 16

.ifndef litewall_black
	litewall_black = 0      ; color put in borders if lw_cols < 10
.endif

; Sets up nametable and begins displaying litewall from NMI.
init_litewall:
	
	; Init PPU
	bit $2002
:       bit $2002
	bpl :-
:       bit $2002
	bpl :-
	lda #$0
	sta $2000
	sta $2001
	
	jsr lw_blacken_palette
	
	; Load nametables
	
	mov $2006,#$20
	mov $2006,#$00
	mov $2000,#$04
.if 0
	lda #4
	jsr @draw_nametable
.else
	ldy #30
	lda #0
:       sta $2007
	dey
	bne :-
	mov $2000,#0
.endif

	lda #$2C
	sta $2006
	lda #$00
	sta $2006
	
	lda #$84
	jsr @draw_nametable
	
	; Clear copy buffer
	ldx #lw_cols_*pitch - 1
	lda #litewall_black
:       sta lw_palsel_copy,x
	sta lw_colors_copy,x
	dex
	bpl :-
	
	; Copy copy loop
	ldx #lw_copy_loop_size - 1
:       lda lw_copy_loop_in_rom,x
	sta lw_copy_loop,x
	dex
	bpl :-
	
	; Init globals
	lda #0
	sta lw_ptr
	sta lw_ready
	sta lw_joy
	sta lw_force_tints
	jsr set_litewall_shape
	
	mov colors_page,#>color_lookup0
	mov tints_page,#>tint_lookup1   ; toggled extra time before being used
	
	lda #$0F
	jsr set_litewall_bgcolor
	
	jsr init_nmi_sync
	
	lda #$80
	sta $2000
	
	rts

@draw_nametable:
	pha
	tax
	
	; Put 4 rows of blocks, each using different tiles
	lda #4
@row:   pha
	ldy #0
:       txa
	clc
	adc @nametable,y
	sta $2007
	iny
	cpy #$60
	bne :-
	
	ldy #$60
:       stx $2007
	dey
	bne :-
	
	txa
	clc
	adc #32
	tax
	
	pla
	sec
	sbc #1
	bne @row
	
	pla
	
	; Clear remaining rows
	ldy #$3C0 - 4*$C0
:       sta $2007
	dey
	bne :-
	
	; Repeat attributes on everything
	ldx #8
:       ldy #0
:       lda @attributes,y
	sta $2007
	iny
	cpy #8
	bne :-
	dex
	bne :--

	rts

@nametable:
	.byte $00
	.byte $02,$0B,$14
	.byte $03,$0C,$15
	.byte $01,$0A,$13
	.byte $03,$0C,$15
	.byte $01,$0A,$13
	.byte $03,$0C,$15
	.byte $01,$0A,$13
	.byte $03,$0C,$15
	.byte $01,$0A,$13
	.byte $02,$0B,$14
	.byte $00
	
	.byte $00
	.byte $05,$0E,$17
	.byte $06,$0F,$18
	.byte $04,$0D,$16
	.byte $06,$0F,$18
	.byte $04,$0D,$16
	.byte $06,$0F,$18
	.byte $04,$0D,$16
	.byte $06,$0F,$18
	.byte $04,$0D,$16
	.byte $05,$0E,$17
	.byte $00
	
	.byte $00
	.byte $08,$11,$1A
	.byte $09,$12,$1B
	.byte $07,$10,$19
	.byte $09,$12,$1B
	.byte $07,$10,$19
	.byte $09,$12,$1B
	.byte $07,$10,$19
	.byte $09,$12,$1B
	.byte $07,$10,$19
	.byte $08,$11,$1A
	.byte $00
	
@attributes:
	.byte $00,$00,$44,$55,$AA,$EE,$FF,$55


; Sets shape to use, from 0 to 3
set_litewall_shape:
	and #lw_max_shapes - 1
	
	tax
	lda @addrs,x
	sta lw_shape_select
	rts

@addrs: .byte $00,$C0,$81,$42,$0C,$CC,$8D,$4E
	.byte $10,$D0,$91,$52,$1C,$DC,$9D,$5E


; Sets background color
set_litewall_bgcolor:
	cmp #$0F
	bne :+
	sta lw_bgcolor
	mov lw_tint_mask,#$FF
	rts

:       pha
	mov lw_tint_mask,#$1F
	pla
	sta lw_bgcolor
	rts


; Stops litewall by disabling NMI and blackening palette
stop_litewall:
	; NMI will overwrite lw_saved_s with different value
	tsx
	stx lw_saved_s
:       cpx lw_saved_s
	beq :-
	
	mov $2000,#0
	mov $2001,#0
	bit $2002
:       bit $2002
	bpl :-
	
lw_blacken_palette:
	lda #$3F
	sta $2006
	ldy #$E0
	sty $2006
:       sta $2007
	iny
	bne :-
	rts


; Waits for NMI then updates litewall with current values in lw_colors_ptr
; and lw_palsel_ptr.
; Preserved: A, X, Y
update_litewall:
	jsr request_litewall_update
	jmp confirm_litewall_update


; Requests that litewall update with current contents of lw_colors_ptr and
; lw_palsel_ptr on the next frame, but returns immediately. You must avoid
; modifying what lw_colors_ptr and lw_palsel_ptr point to until update occurs.
; Call confirm_litewall_update to wait until this update actually occurs.
; Preserved: A, X, Y
request_litewall_update:
	jsr confirm_litewall_update
	
	; Update pointers in loop in RAM
	lda lw_palsel_ptr
	sta lw_copy_loop + $01
	clc
	adc #lw_copy_loop_half
	sta lw_copy_loop + $0D
	lda lw_palsel_ptr + 1
	sta lw_copy_loop + $02
	adc #0
	sta lw_copy_loop + $0E
	
	lda lw_colors_ptr
	sta lw_copy_loop + $07
	clc
	adc #lw_copy_loop_half
	sta lw_copy_loop + $13
	lda lw_colors_ptr + 1
	sta lw_copy_loop + $08
	adc #0
	sta lw_copy_loop + $14
	
	; Set ready flag
	dec lw_ready
	
	rts


; Waits for update to occur, if one was initiated via update_litewall_async.
; If none is pending, returns immediately.
; Preserved: A, X, Y
confirm_litewall_update:
:       inc lw_rand_seed
	bit lw_ready
	bmi :-
	rts


.align 256
sprites:
	lw_overflow_time = 3 ; 2-4 work
	
	.res lw_overflow_time*4,$F0
	.res $100 - lw_overflow_time*4,0

.include "lw_palette.s"

; Definition of shape tiles. begin_litewall_shape, then 9 x def_litewall_tile

.macro begin_litewall_shape
	.res $50,0
.endmacro

.macro def_litewall_tile Bytes
	.res 8,0
	.byte Bytes
	
	.byte Bytes
	.byte Bytes

	.byte Bytes
	.res 8,0
.endmacro
