; SNES version of litewall
;
; ca65 -I snes -I common snes/litewall.s
; ld65 -C snes/smc.cfg -o litewall.smc snes/litewall.o

SNES = 1
.p816
sram_begin = $700000
.define ROMTITLE "LITEWALL"

; Leave space at beginning for my devcart
.res $200

; Put at beginning of 65816 code that may be called
; from emulation mode. Saves current mode and register
; sizes, then sets 16-bit X and Y.
.macro native_routine
	clc
	xce
	php
	i16
.endmacro

.macro native_rts
	plp
	xce
	rts
	.i8
.endmacro

.include "sneshw.inc"
.include "macros.inc"
.include "65816_macros.inc"
.include "common.inc"
.include "ppu.s"
.include "main.s"

reset:
	ldx #$FF
	txs
	
	; Related to my devcart
	;mov $707FFC,#<$FE00
	;mov $707FFD,#>$FE00
	movw $707FEA,#native_nmi
	movw $707FFA,#nmi
	
	jsr clear_ram
	jmp main

; Clears most of RAM, except what's currently on the stack.
clear_ram:
	lda #0
	ldx #0
:       sta 0,x
	sta $200,x
	sta $300,x
	sta $400,x
	sta $500,x
	sta $600,x
	inx
	bne :-
	
	; Clear everything below stack
	tsx
	inx
:       dex
	sta $100,x
	bne :-
	
	rts

; Switch to emulation mode and invoke NES nmi
native_nmi:
	; Save 16-bit registers, since emulation mode changes
	; upper 8 bits
	ai16
	pha
	phx
	phy
	
	sec
	xce
	pea :+
	php
	jmp nmi
:       clc
	xce
	
	ai16
	ply
	plx
	pla
	.a8
	rti

.segment "HEADER"
	.byte ROMTITLE
	.res $15 - .strlen(ROMTITLE),$20
	.byte $20       ; LoROM
	.byte $02       ; SRAM
	.byte $08       ; 128K ROM
	.byte $03       ; 8K SRAM
	.byte 0,0,0
	.word $AAAA,$5555 ; dummy checksum and complement

.segment "VECTORS"
	.res $0A
	.word native_nmi
	
	.align $10
	.res $0A
	.word nmi
	.word reset
