; Initializes litewall

; All code executed natively
.i16

; Vertical centering of grid, adjust as desired
lw_top = 5

; Size of a nametable
nt_size = $400

; Place for nametables
lw_chr = $4000

lw_init:
	jsr init_ppu_regs
	jsr blacken_palette
	
	jsr lw_load_chr
	
	; Write 8 nametables, each using $20 tiles out of $100
	ldx #lw_nametable
	lda #4
:       pha
	phx
	jsr lw_load_nametable
	a16
	pla
	clc
	adc #nt_size
	tax
	a8
	pla
	clc
	adc #$20
	bcc :-

	jsr snes_wait_vbl
	
	; CHR pointers
	mov BG12NBA,#(lw_chr >> 12)*$11
	mov BG34NBA,#(lw_chr >> 12)*$11
	
	; Nametable pointers
	mov BG1SC,#>lw_nametable
	mov BG2SC,#>lw_nametable
	mov BG3SC,#>lw_nametable
	mov BG4SC,#>lw_nametable
	
	; Y scroll positions
	mov BG1VOFS,#-lw_top
	mov BG1VOFS,#0

	mov BG2VOFS,#-lw_top-28
	mov BG2VOFS,#0

	mov BG3VOFS,#-lw_top-28*4
	mov BG3VOFS,#0

	mov BG4VOFS,#-lw_top-28*5
	mov BG4VOFS,#0

	; Enable all four screens
	mov TM,#$0F
	mov INIDISP,#$0F
	
	rts

lw_load_chr:
	; Load tiles
	mov VMAIN,#$80  ; word video writes
	movw VMADDL,#lw_chr
	
	ldx #0
:
	; Convert from NES to SNES format
	ldy #8
:       lda tiles,x
	sta VMDATAL
	lda tiles+8,x
	sta VMDATAH
	inx
	dey
	bne :-
	
	a16
	txa
	clc
	adc #8
	tax
	a8
	
	cpx #tiles_size
	bne :--
	
	rts

lw_load_nametable:
	pha
	phx
	jsr lw_clear_nametable
	
	plx
	stx VMADDL
	mov VMAIN,#$80  ; word video writes
	
	pla
	sta temp
	ldx #0
:       lda light_row,x
	clc
	adc temp
	sta VMDATAL
	lda light_row_attr,x
	sta VMDATAH
	inx
	cpx #$20 * (3 + 4 + 3)
	bne :-
	
	rts

lw_clear_nametable:
	stx VMADDL
	ldy #0
	ldx #$400
:       sty VMDATAL
	dex
	bne :-
	rts

.macro begin_litewall_shape
	.res $50,0
.endmacro

.macro def_litewall_tile Bytes
	.res 8,0
	.byte Bytes
	
	.byte Bytes
	.byte Bytes

	.byte Bytes
	.res 8,0
.endmacro

tiles:
	.include "shapes.s"

tiles_size = * - tiles

light_row:
.repeat 2,i
	.byte $00
	.byte $03,$0C,$15
	.byte $01,$0A,$13
	.byte $02,$0B,$14
	.byte $03,$0C,$15
	.byte $01,$0A,$13
	.byte $02,$0B,$14
	.byte $03,$0C,$15
	.byte $01,$0A,$13
	.byte $02,$0B,$14
	.byte $03,$0C,$15
	.byte $00
	
	.byte $00
	.byte $06,$0F,$18
	.byte $04,$0D,$16
	.byte $05,$0E,$17
	.byte $06,$0F,$18
	.byte $04,$0D,$16
	.byte $05,$0E,$17
	.byte $06,$0F,$18
	.byte $04,$0D,$16
	.byte $05,$0E,$17
	.byte $06,$0F,$18
	.byte $00
	
	.byte $00
	.byte $09,$12,$1B
	.byte $07,$10,$19
	.byte $08,$11,$1A
	.byte $09,$12,$1B
	.byte $07,$10,$19
	.byte $08,$11,$1A
	.byte $09,$12,$1B
	.byte $07,$10,$19
	.byte $08,$11,$1A
	.byte $09,$12,$1B
	.byte $00

	.if i = 0
	.res $20*4,0
	.endif
.endrepeat

light_row_attr:
.repeat 3
	.byte $00
	.byte $00,$00,$00
	.byte $00,$00,$00
	.byte $00,$00,$00
	.byte $04,$04,$04
	.byte $04,$04,$04
	.byte $04,$04,$04
	.byte $08,$08,$08
	.byte $08,$08,$08
	.byte $08,$08,$08
	.byte $0C,$0C,$0C
	.byte $00
.endrepeat

	.res $20*4,0

.repeat 3
	.byte $00
	.byte $10,$10,$10
	.byte $10,$10,$10
	.byte $10,$10,$10
	.byte $14,$14,$14
	.byte $14,$14,$14
	.byte $14,$14,$14
	.byte $18,$18,$18
	.byte $18,$18,$18
	.byte $18,$18,$18
	.byte $1C,$1C,$1C
	.byte $00
.endrepeat

.i8
